'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _q = require('q');

var _q2 = _interopRequireDefault(_q);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _feathers = require('feathers');

var _feathers2 = _interopRequireDefault(_feathers);

var log = (0, _debug2['default'])('mysam:plugins');

exports['default'] = function (global) {
  return function () {
    var app = this;
    var plugins = app.service('plugins');
    var loadPlugin = function loadPlugin(moduleName) {
      try {
        var pkgPath = require.resolve(_path2['default'].join(moduleName, 'package.json'));

        var pkg = require(pkgPath);
        var config = pkg.mysam;

        if (!config) {
          throw new Error(moduleName + ' is not a Mysam plugin.');
        }

        log('Found plugin ' + pkg.name);

        var dirname = _path2['default'].dirname(pkgPath);
        var staticPath = _path2['default'].join(dirname, config['public'] || '.');

        log('Setting up ' + staticPath + ' at /' + pkg.name);

        app.use('/' + pkg.name, _feathers2['default']['static'](staticPath));

        return _q2['default'].ninvoke(plugins, 'create', pkg).then(function () {
          var pluginLoader = require(moduleName);
          pluginLoader(app);
        });
      } catch (e) {
        log('Not loading plugin module ' + moduleName);
        return (0, _q2['default'])(null);
      }
    };

    return _q2['default'].nfcall(_child_process.exec, 'npm ls --depth 0 --parseable ' + (global ? '-g' : '')).then(function (stdout) {
      var modules = stdout.toString().trim().split('\n');

      return _q2['default'].all(modules.map(function (path) {
        return loadPlugin(path);
      }));
    }).fail(function (error) {
      return console.error(error.stack);
    });
  };
};

module.exports = exports['default'];