'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});
exports.addPos = addPos;
exports.extract = extract;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _q = require('q');

var _q2 = _interopRequireDefault(_q);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

var _naturalBrain = require('natural-brain');

var _naturalBrain2 = _interopRequireDefault(_naturalBrain);

var _pos = require('pos');

var _pos2 = _interopRequireDefault(_pos);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _extractor = require('../extractor');

var _extractor2 = _interopRequireDefault(_extractor);

var log = (0, _debug2['default'])('mysam:classifier');
var lexer = new _pos2['default'].Lexer();
var tagger = new _pos2['default'].Tagger();

function addPos(hook, next) {
  var data = hook.result;
  var tokens = lexer.lex(data.input);
  var tags = tagger.tag(tokens).map(function (touple) {
    return touple[1];
  });

  data.pos = { tokens: tokens, tags: tags };
  next();
}

function extract(hook, next) {
  var data = hook.result;
  var e = new _extractor2['default'](lexer.lex(data.text), data.tags || []);

  data.extracted = e.extract(data.pos.tokens);
  next();
}

exports['default'] = function () {
  return {
    after: {
      create: [addPos, extract]
    },
    create: function create(data, params, callback) {
      log('Classifying', data);

      if (this.retrain) {
        log('Retraining Neural network');
        this.classifier.retrain();
        this.retrain = false;
      }

      var input = data.input;

      var id = _crypto2['default'].createHash('md5').update(input).digest('hex');
      var action = this.classifier.classify(input);
      var classifications = this.classifier.getClassifications(input);

      _q2['default'].ninvoke(this.actions, 'get', action).then(function (action) {
        return _lodash2['default'].extend(action, { id: id, input: input, classifications: classifications });
      }).then(function (result) {
        log(result);
        return result;
      }).fail(function (err) {
        return log(err.stack);
      }).nodeify(callback);
    },

    add: function add(action) {
      log('Adding action to training', action);
      this.classifier.addDocument(action.text, '' + action._id);
      this.retrain = true;
    },

    setup: function setup(app) {
      var _this = this;

      var add = this.add.bind(this);

      // We want to classify every word
      _naturalBrain2['default'].disableStopWords();

      this.classifier = new _naturalBrain2['default']({
        iterations: 2000
      });

      this.actions = app.service('actions');
      this.actions.on('created', add);
      //this.actions.on('removed', action => {
      //
      //});

      _q2['default'].ninvoke(this.actions, 'find').then(function (list) {
        list.forEach(add);
        return list;
      }).then(function (list) {
        if (list.length) {
          _this.classifier.train();
          _this.retrain = false;
        }
      }).fail(function (error) {
        return log('ERROR: ', error.stack);
      });
    }
  };
};